% =========================================================================
% A wrapper experiment script for the robust Logistic Regression algorithm
% =========================================================================


% start with nice & clean workspace
clear all
close all 

%s = RandStream('mcg16807','Seed',121121); RandStream.setDefaultStream(s);

ITER      = 10;        % Inner loop, corresponds to train/test spliting.
EXP_RANGE = 5;         % Outer loop, corresponds to level of label noise.
NoiseType = 'asym';    % 'sym' or 'asym' 

% Storing generalisation errors
e_lr      = zeros(ITER, EXP_RANGE); 
e_rlr     = zeros(ITER, EXP_RANGE);
e_rmlr    = zeros(ITER, EXP_RANGE); 

% toy data: UCI Boston
load boston
DPnt  = size(x,1);
TrPnt = floor(0.8 * DPnt);

for i = 1:EXP_RANGE     
    % define label flipping probability
    FLIPP = 0.1 * (i-1);
        
    for j = 1:ITER
        
        perm = randperm(DPnt);
        tr = perm(1:TrPnt);
        ts = perm(TrPnt+1:end);
        
        X  = addbias(x(tr,:));   % training set
        Xt = addbias(x(ts,:));   % test set
        yt = y(tr,:);   % training labels
        ys = y(ts,:);   % test labels
        
        
        % output current loop/repetition
        fprintf(1, 'i=%d, j=%d\n', i, j);
        
        % add label noise
        [yz fdz] = injectLabelNoise(yt, fd(tr), NoiseType, FLIPP);
       
        % estimate models
        opts.estG = true; opts.verbose = false;
        [w_lr  g_lr  llh_lr]  = rlr(zeros(size(X,2),1), eye(2), X,... 
                                    castLabel(yz,0), @l2, opts);
        [w_rlr g_rlr llh_rlr] = rlr(zeros(size(X,2),1), [.8 .2;.2 .8], X,... 
                                    castLabel(yz,0), @l2, opts);                                       
        [w_rmlr g_rmlr]       = rmlr(zeros(size(X,2),2), [.8 .2;.2 .8], X,...
                                    castLabel(yz,2), @l2, opts);

        % evaluate models
        [p d t e_lr(j,i)]   = evalLR(Xt, ys, w_lr);  
        [p d t e_rlr(j,i)]  = evalLR(Xt, ys, w_rlr);  
        [p d t e_rmlr(j,i)] = evalLR(Xt, ys, w_rmlr);
    end
end

disp('Generalisation errors: 0%-40% asymmetric noise');
disp('Errors (traditional LR)'); 
disp(mean(e_lr));

disp('Errors (robust LR)'); 
disp(mean(e_rlr));

disp('Errors (robust mLR)'); 
disp(mean(e_rmlr));
