% Function to evaluate model of logistic regression family
% Author: Jakramate Bootkrajang
% Input : x  = Samples to be classified
%         y  = Target value
%         w  = Weight vector for Sigmoid/Softmax
%         fd = Label flipping indicator
% Return: prob  = probability of predicted class
%         pred  = predicted class
%         eIdx  = misclassification vector
%         eRate = misclassification rate


function [prob pred eIdx eRate] = evalLR(x, y, w)

y = castLabel(y,2);

if (size(x,2) ~= size(w,1))
    error('input and weight vector have different length missing bias?');
end

TPNT = size(x,1);                % total number of test samples 
CLS  = size(w,2) + 1;            % determine number of classes

if (CLS == 2)   
    t      = (x*w);
    postY  = sigmoid(t);         % calculate the sigmoid
    postY  = [1-postY postY]';        
else   
    postY = softmax(x * w)';     % calculate the softmax        
end

[prob pred] = max(postY); 
eIdx        = (pred' ~= y);
eRate       = sum(eIdx)/TPNT;

